version 9.1
	mata:
	mata clear
	mata set matastrict on
	
	// Simply a caller for the Needleman-Wunsch Algorithm
	void ///
	  sqom(string scalar varlist, real scalar indelcost, string scalar standard, real scalar k, real scalar subcost, real scalar mod )
	{
		real matrix X      // Data-Matrix
		real matrix D      // Distance-Matrix
		real rowvector R   // 1st Selected Sequence (Row of Levensthein)
		real rowvector C   // 2nd Selected Sequence (Col of Levensthein)
		real scalar i
		real scalar j
		real scalar cut
		void SQdist

		st_view(X=.,.,tokens(varlist))

		D = J(rows(X),rows(X),0)

		// Call NeedlemanWunsch with Fixed Subcosts
		if (subcost>0) {
			for (i=1;i<=rows(X)-1;i++) {
				for (j=i+1;j<=rows(X);j++) {
					R = X[i,1..sqlength(X[i,.])]
					C = X[j,1..sqlength(X[j,.])]
					if (k==0) {
						D[j,i] = needlemanwunschexactfixed(R,C,indelcost,standard,subcost)
					}
					else {
						D[j,i] = needlemanwunschapproxfixed(R,C,indelcost,k,standard,subcost)
					}
				}
			}
			st_matrix("SQdist",(makesymmetric(D)))
		}

		// Call NeedlemanWunsch with Fixed Subcosts
		else if (subcost==-1) {
			for (i=1;i<=rows(X)-1;i++) {
				for (j=i+1;j<=rows(X);j++) {
					R = X[i,1..sqlength(X[i,.])]
					C = X[j,1..sqlength(X[j,.])]
					if (k==0) {
						D[j,i] = needlemanwunschexactrawdist(R,C,indelcost,standard)
					}
					else {
						D[j,i] = needlemanwunschapproxrawdist(R,C,indelcost,k,standard)
					}
				}
			}
			st_matrix("SQdist",(makesymmetric(D)))
		}

		// Call  NeedlemanWunsch with Subcost-Matrix
		else {
			for (i=1;i<=rows(X)-1;i++) {
				for (j=i+1;j<=rows(X);j++) {
					R = X[i,1..sqlength(X[i,.])]
					C = X[j,1..sqlength(X[j,.])]
					if (k==0) {
						D[j,i] = needlemanwunschexactmatrix(R,C,indelcost,standard,mod)
					}
					else {
						D[j,i] = needlemanwunschapproxmatrix(R,C,indelcost,k,standard,mod)
					}
				}
			}
			st_matrix("SQdist",(makesymmetric(D)))
		}
	}

	// Mata Function to Perform Needleman-Wunsch Algorithm fixed Subcosts
	real scalar needlemanwunschexactfixed(real rowvector R, real rowvector C, real scalar indelcost, string scalar standard, real scalar subcost)
	{

		real matrix L       // Levensthein-Matrix
		real rowvector M    // Vector of Values to be Minimized
		real scalar i
		real scalar j
		


		// Initialize Levensthein-Matrix
		L = J(cols(R)+1,cols(C)+1,0)

		// Initialize First Row/Col of Levensthein
		for (i=2;i<=cols(R)+1;i++) {
			L[i,1]=L[i-1,1]+indelcost
		}
		for (j=2;j<=cols(C)+1;j++) {
			L[1,j]=L[1,j-1]+indelcost
		}
		
		// Step thru the Levensthein 
		for (i=2;i<=cols(R)+1;i++) {
				for (j=2;j<=cols(C)+1;j++) {
					M = L[i-1,j-1]+ (R[1,i-1]==C[1,j-1]? 0 : subcost),L[i-1,j]+indelcost,L[i,j-1]+indelcost
					L[i,j]=min(M)
			}
		}
		return((L[cols(R)+1,cols(C)+1])/(standard=="longest" ? max((cols(R),cols(C))):1))
	}


	// Mata Function to Perform Needleman-Wunsch Algorithm with rawdistance-subcosts
	real scalar needlemanwunschexactrawdist(real rowvector R, real rowvector C, real scalar indelcost, string scalar standard)
	{

		real matrix L       // Levensthein-Matrix
		real rowvector M    // Vector of Values to be Minimized
		real scalar i
		real scalar j

		// Initialize Levensthein-Matrix
		L = J(cols(R)+1,cols(C)+1,0)

		// Initialize First Row/Col of Levensthein
		for (i=2;i<=cols(R)+1;i++) {
			L[i,1]=L[i-1,1]+indelcost
		}
		for (j=2;j<=cols(C)+1;j++) {
			L[1,j]=L[1,j-1]+indelcost
		}
		
		// Step thru the Levensthein 
		for (i=2;i<=cols(R)+1;i++) {
				for (j=2;j<=cols(C)+1;j++) {
					M = L[i-1,j-1]+ abs(R[1,i-1]-C[1,j-1]),L[i-1,j]+indelcost,L[i,j-1]+indelcost
					L[i,j]=min(M)
			}
		}
		return((L[cols(R)+1,cols(C)+1])/(standard=="longest" ? max((cols(R),cols(C))):1))
	}

	// Mata Function to Perform Needleman-Wunsch Algorithm with Subcost Matrix
	real scalar needlemanwunschexactmatrix(real rowvector R, real rowvector C, real scalar indelcost,string scalar standard,mod)
	{

		real matrix L       // Levensthein-Matrix
		real rowvector M    // Vector of Values to be Minimized
		real scalar i
		real scalar j
		
		// Initialize Levensthein-Matrix
		L = J(cols(R)+1,cols(C)+1,0)

		// Initialize First Row/Col of Levensthein
		for (i=2;i<=cols(R)+1;i++) {
			L[i,1]=L[i-1,1]+indelcost
		}
		for (j=2;j<=cols(C)+1;j++) {
			L[1,j]=L[1,j-1]+indelcost
		}
		
		// Step thru the Levensthein 
		for (i=2;i<=cols(R)+1;i++) {
			for (j=2;j<=cols(C)+1;j++) {
//				M = L[i-1,j-1]+subcosthash(R,C,i-1,j-1,mod),L[i-1,j]+indelcost,L[i,j-1]+indelcost
				M = L[i-1,j-1]+subcost(R,C,i-1,j-1),L[i-1,j]+indelcost,L[i,j-1]+indelcost
				L[i,j]=min(M)
			}
		}
		return((L[cols(R)+1,cols(C)+1])/(standard=="longest" ? max((cols(R),cols(C))):1))
	}

    // Mata Function to perfom NW-approximation wit fixed subcost
    real scalar needlemanwunschapproxfixed(real rowvector R, real rowvector C, real scalar indelcost, real scalar paramK, string scalar standard, real scalar subcost)
    {
            real scalar distance            //return value
            distance = 0

            real scalar inser               //auxiliary variable
            real scalar del                 //auxiliary variable
            real scalar help1               //auxiliary variable
            real scalar help2               //auxiliary variable
            real scalar help3               //auxiliary variable
            real scalar index1              //auxiliary variable
            real scalar index2              //auxiliary variable
            real scalar i
            real scalar j

            //Length of sequences
            real scalar iLength
            real scalar jLength
            iLength = sqlength(R)
            jLength = sqlength(C)


            //Dimension of Levensthein-matrix
            real scalar matrixDim
            real vector v
            v = iLength, jLength
            matrixDim = max(v)+1

            //Boundary of Levensthein-matrix
            real scalar bound
            bound = (iLength/jLength + jLength/iLength)/sqrt(iLength*iLength+jLength*jLength)
            bound = paramK * bound / sqrt(2)

            //Declaration and initialisation of Levensthein-matrix
            //Levensthein-matrix saved as a vector
            //Levensthein-matrix initialized with infinity
            real matrix L
            L = J(matrixDim*matrixDim, 1, .)
            L[1,1]=0
            for(i=1; i<matrixDim; i++) {
                L[i+1,1]=L[i,1] + indelcost
                if(i<matrixDim) {
                L[i*matrixDim+1,1]=L[((i-1)*matrixDim)+1,1] + indelcost
                }
            }

            //Control variables for loop for the second sequence
            real scalar j1
            j1=1
            real scalar j2
            j2=jLength +1
            real scalar j3
            j3=jLength +1

            //1. loop: the first sequence
            for(i=1; i<iLength+1; i++) {
                help1 = (i)/iLength
                help2 = (jLength+1)*(help1-bound)
                help3 = (jLength+1)*(help1+bound)

                //Values of control variables for loop for the second sequence
                if(help2 <= 2) {
                    j1 = 1
                }
                else {
                    j1 = floor(help2)
                }

                if(help3>=jLength+1) {
                    j2 = jLength +1
                }
                else {
                    j2 = floor(help3)
                }

            //coordinates for Levensthein-matrix(vector)
                index1 = (i-1)*matrixDim + j1
                index2 = index1 + matrixDim

            //2. loop: the second sequence
                for(j=j1; j<j2; j++) {
                //substitutioncost
                    distance = (R[1,i]==C[1,j]? 0 : subcost)
                    distance = distance + L[index1, 1]
                    index1++

                //insertion
                    if(j>j1 || j==1) {
                        inser = indelcost + L[index2, 1]
                        if(distance>inser) {
                                distance= inser
                            }
                    }
                    index2++
                    //deletion
                    if(j<j3) {
                        del = indelcost+L[index1, 1]
                            if(distance>del) {
                                distance=del
                            }
                    L[index2,1]=distance
                }
                j3=j2
                }
            }
		return(distance/(standard=="longest" ? max((cols(R),cols(C))):1))
    }


	// Mata Function to perfom NW-approximation with rawdistnace-subcosts
    real scalar needlemanwunschapproxrawdist(real rowvector R, real rowvector C, real scalar indelcost, real scalar paramK, string scalar standard)
    {
            real scalar distance            //return value
            distance = 0

            real scalar inser               //auxiliary variable
            real scalar del                 //auxiliary variable
            real scalar help1               //auxiliary variable
            real scalar help2               //auxiliary variable
            real scalar help3               //auxiliary variable
            real scalar index1              //auxiliary variable
            real scalar index2              //auxiliary variable
            real scalar i
            real scalar j

            //Length of sequences
            real scalar iLength
            real scalar jLength
            iLength = sqlength(R)
            jLength = sqlength(C)


            //Dimension of Levensthein-matrix
            real scalar matrixDim
            real vector v
            v = iLength, jLength
            matrixDim = max(v)+1

            //Boundary of Levensthein-matrix
            real scalar bound
            bound = (iLength/jLength + jLength/iLength)/sqrt(iLength*iLength+jLength*jLength)
            bound = paramK * bound / sqrt(2)

            //Declaration and initialisation of Levensthein-matrix
            //Levensthein-matrix saved as a vector
            //Levensthein-matrix initialized with infinity
            real matrix L
            L = J(matrixDim*matrixDim, 1, .)
            L[1,1]=0
            for(i=1; i<matrixDim; i++) {
                L[i+1,1]=L[i,1] + indelcost
                if(i<matrixDim) {
                L[i*matrixDim+1,1]=L[((i-1)*matrixDim)+1,1] + indelcost
                }
            }

            //Control variables for loop for the second sequence
            real scalar j1
            j1=1
            real scalar j2
            j2=jLength +1
            real scalar j3
            j3=jLength +1

            //1. loop: the first sequence
            for(i=1; i<iLength+1; i++) {
                help1 = (i)/iLength
                help2 = (jLength+1)*(help1-bound)
                help3 = (jLength+1)*(help1+bound)

                //Values of control variables for loop for the second sequence
                if(help2 <= 2) {
                    j1 = 1
                }
                else {
                    j1 = floor(help2)
                }

                if(help3>=jLength+1) {
                    j2 = jLength +1
                }
                else {
                    j2 = floor(help3)
                }

            //coordinates for Levensthein-matrix(vector)
                index1 = (i-1)*matrixDim + j1
                index2 = index1 + matrixDim

            //2. loop: the second sequence
                for(j=j1; j<j2; j++) {

				//substitutioncost
				    distance = abs(R[1,i]-C[1,j])
				    distance = distance + L[index1, 1]
                    index1++

                //insertion
                    if(j>j1 || j==1) {
                        inser = indelcost + L[index2, 1]
                        if(distance>inser) {
                                distance= inser
                            }
                    }
                    index2++
                    //deletion
                    if(j<j3) {
                        del = indelcost+L[index1, 1]
                            if(distance>del) {
                                distance=del
                            }
                    L[index2,1]=distance
                }
                j3=j2
                }
            }
		return(distance/(standard=="longest" ? max((cols(R),cols(C))):1))
    }


    // Mata Function to perfom NW-approximation with subcost-matrix
    real scalar needlemanwunschapproxmatrix(real rowvector R, real rowvector C, real scalar indelcost, real scalar paramK, string scalar standard,real scalar mod)
    {
            real scalar distance            //return value
            distance = 0

            real scalar inser               //auxiliary variable
            real scalar del                 //auxiliary variable
            real scalar help1               //auxiliary variable
            real scalar help2               //auxiliary variable
            real scalar help3               //auxiliary variable
            real scalar index1              //auxiliary variable
            real scalar index2              //auxiliary variable
            real scalar i
            real scalar j

            //Length of sequences
            real scalar iLength
            real scalar jLength
            iLength = sqlength(R)
            jLength = sqlength(C)


            //Dimension of Levensthein-matrix
            real scalar matrixDim
            real vector v
            v = iLength, jLength
            matrixDim = max(v)+1

            //Boundary of Levensthein-matrix
            real scalar bound
            bound = (iLength/jLength + jLength/iLength)/sqrt(iLength*iLength+jLength*jLength)
            bound = paramK * bound / sqrt(2)

            //Declaration and initialisation of Levensthein-matrix
            //Levensthein-matrix saved as a vector
            //Levensthein-matrix initialized with infinity
            real matrix L
            L = J(matrixDim*matrixDim, 1, .)
            L[1,1]=0
            for(i=1; i<matrixDim; i++) {
                L[i+1,1]=L[i,1] + indelcost
                if(i<matrixDim) {
                L[i*matrixDim+1,1]=L[((i-1)*matrixDim)+1,1] + indelcost
                }
            }

            //Control variables for loop for the second sequence
            real scalar j1
            j1=1
            real scalar j2
            j2=jLength +1
            real scalar j3
            j3=jLength +1

            //1. loop: the first sequence
            for(i=1; i<iLength+1; i++) {
                help1 = (i)/iLength
                help2 = (jLength+1)*(help1-bound)
                help3 = (jLength+1)*(help1+bound)

                //Values of control variables for loop for the second sequence
                if(help2 <= 2) {
                    j1 = 1
                }
                else {
                    j1 = floor(help2)
                }

                if(help3>=jLength+1) {
                    j2 = jLength +1
                }
                else {
                    j2 = floor(help3)
                }

            //coordinates for Levensthein-matrix(vector)
                index1 = (i-1)*matrixDim + j1
                index2 = index1 + matrixDim

            //2. loop: the second sequence
                for(j=j1; j<j2; j++) {
                //substitutioncost
//                    distance = subcosthash(R, C, i, j,mod)
                    distance = subcost(R, C, i, j)
                    distance = distance + L[index1, 1]
                    index1++

                //insertion
                    if(j>j1 || j==1) {
                        inser = indelcost + L[index2, 1]
                        if(distance>inser) {
                                distance= inser
                            }
                    }
                    index2++
                    //deletion
                    if(j<j3) {
                        del = indelcost+L[index1, 1]
                            if(distance>del) {
                                distance=del
                            }
                    L[index2,1]=distance
                }
                j3=j2
                }
            }
		return(distance/(standard=="longest" ? max((cols(R),cols(C))):1))
    }

		
	// Mata Function to extract the sequence-length
	real scalar sqlength(transmorphic rowvector X)
	{
		real scalar i 
		real scalar col

			for (i=1;i<=cols(X);i++) {
				if (X[1,i] != missingof(X))  col=i 
			}
			return(col)
	}

	// Mata Function to extract the substitution costs from subcost-matrix
	real scalar subcost(real rowvector R, real rowvector C, real scalar a, real scalar b)
	{
		real matrix SQsubcost
		real matrix SQlevels
		string rowvector X
		string rowvector Y 
		real scalar i 
		real scalar row
		real scalar col

		SQsubcost = st_matrix("SQsubcost")
		X = strofreal(R)
		Y = strofreal(C)
		SQlevels = st_matrixrowstripe("SQsubcost")
		for (i=1;i<=rows(SQsubcost);i++) {
			if (SQlevels[i,2] == X[1,a]) row=i
			if (SQlevels[i,2] == Y[1,b]) col=i
		}
		return(SQsubcost[row,col])
	}


	// Mata Function to extract the substitution costs from subcost-matrix via hash-table
	// temporary out of order 
	real scalar subcosthash(real rowvector R, real rowvector C, real scalar a, real scalar b, real scalar mod)
	{
		stata("SQGetCost "  +  strofreal(R[1,a])  +  "  " + strofreal(C[1,b]) +  " " + strofreal(mod) +  " ")
		return(st_numscalar("SQsubcostvalue"))
	}

	// Compile into a libary
	mata mlib create lsq, replace
	mata mlib add ///
	  lsq sqom() ///
	  needlemanwunschexactfixed() needlemanwunschexactrawdist() needlemanwunschexactmatrix() ///
	  needlemanwunschapproxfixed() needlemanwunschapproxrawdist() needlemanwunschapproxmatrix() ///
	  sqlength() subcost() subcosthash()
	mata mlib index
	
end
exit
	

