*! version 1.0.0 March 20, 2006 @ 18:26:50
*! Sumarize information on (person specific) sequences
program define sqparcoord
version 9
	syntax [if] [in] ///
	  [, I(varname) T(varname) sq(varname) ranks(numlist) so * offset(string) wlines(string) ]

	// Sq-Data
	capture _sqset `sq' `i' `t'
	if _rc {
		di as error "data not declared as SQ-data; use -sqset-"
		exit 9
	}


	// if/in
	if "`if'" != "" {
		tokenize "`if'", parse(" =+-*/^~!|&<>(),.")
		while "`1'" != "" {
			capture confirm variable `1'
			if !_rc {
				local iflist  "`iflist' `1'"
			}
			macro shift
		}
	}
	if "`iflist'" != "" CheckConstant `iflist', stop
	marksample touse 


	quietly {

		preserve
		keep if `touse'
		
		// Stretch Scale for option SO
		if "`so'" == "so" {
			tempvar order howmany
			by `_dta[SQiis]' (`_dta[SQtis]'), sort: ///
			  keep if `_dta[SQis]' ~= `_dta[SQis]'[_n-1]
			by `_dta[SQiis]' (`_dta[SQtis]'), sort: gen `howmany' = _N
			expand 2 if `howmany' == 1
			by `_dta[SQiis]' (`_dta[SQtis]'), sort: replace `_dta[SQtis]' = _n
			by `_dta[SQiis]', sort: gen `order' = (_n-1)/(_N-1)
		}
		
		// Sequence-Frequency Data
		tempvar n
		keep `_dta[SQiis]' `_dta[SQtis]' `_dta[SQis]' `order' 
		reshape wide `_dta[SQis]' `order' , i(`_dta[SQiis]') j(`_dta[SQtis]')
		bysort `_dta[SQis]'*: gen `n' = _N
		bysort `_dta[SQis]'*: keep if _n==1

		// Option ranks
		if "`ranks'" != "" KeepRanks `n', ranks(`ranks')

		// Prepare random number for option jitter
		if "`offset'" != "" {
			tempvar noise
			gen `noise' = uniform()*`offset'-`offset'/2
		}
		
		// Reshape long
		reshape long `_dta[SQis]' `order', i(`_dta[SQiis]') j(`_dta[SQtis]')

		// Relabel timevar for Option SO
		if "`so'" != "" {
			replace `_dta[SQtis]' = `order'
			label variable `_dta[SQtis]' "Order"
		}

		// Add random noise
		if "`offset'" != "" replace `_dta[SQis]' = `_dta[SQis]' + `noise'

		// Graphs
		sort  `_dta[SQiis]' `_dta[SQtis]' 
		if "`wlines'" != "" GraphWithWlines `n', wlines(`wlines') `options'
		else graph twoway line `_dta[SQis]' `_dta[SQtis]', c(L) `options'
	}
end
		

// Subprogram for graphs with lines proportional to frequency
program GraphWithWlines
	syntax varname, wlines(string) *
	sum `varlist', meanonly
	local max = r(max)
	levelsof `varlist', local(K)
	foreach k of local K {
		tempvar sq`k'
		gen `sq`k'' =  `_dta[SQis]' if `varlist' == `k'
		local lwidth = `k'/`max' * `wlines'
		local line "`line' (line `sq`k'' `_dta[SQtis]', c(L) lstyle(p1) lwidth(`lwidth') `options')"
	}
	graph twoway `line', legend(off)
end

	
// Selects Ranks according to rank-Options
program KeepRanks
	syntax varname, ranks(string) 
	tempvar rank select
	sort `1'
	gen `rank' = (_N+1) - _n
	gen `select' = 0
	foreach r of local ranks {
		replace `select' = 1 if `rank'  == `r'
	}
	keep if `select'
end	

program CheckConstant, rclass
	syntax varlist(default=none) [, stop]
	sort `_dta[SQiis]'
	foreach var of local varlist {
		capture by `_dta[SQiis]': assert `var' == `var'[_n-1] if _n != 1
		if _rc & "`stop'" == "" {
			di as res "`var'" as text " is not constant over time; not used"
			local varlist: subinstr local varlist "`var'" "", word
		}
		if _rc & "`stop'" != "" {
			di as error "`var' is not constant over time"
			exit 9
		}
		if "`stop'" == "" {
			return local checked "`varlist'"
		}
	}
end

