*! version 1.0.0 January 4, 2006 @ 11:34:25
*! SQset data

program sqset
version 9
	syntax [varlist(max=3 default=none)] [,	CLEAR ]
	
	if "`clear'" != "" { 
		syntax [, CLEAR]
		Clear
		exit
	}

	if "`varlist'"=="" {
		Query
		Check
		exit
	}
	
	syntax varlist(min=3 max=3)
	
	// Sequence 
	char _dta[SQis] `1' 
	char _dta[SQiis] `2' 
	char _dta[SQtis] `3'
	Check
end

program define Clear
	char _dta[SQtis]
	char _dta[SQiis]
	char _dta[SQis]
end
	
program define Query
	local sqvar `_dta[SQis]'
	local ivar `_dta[SQiis]'
	local tvar `_dta[SQtis]'

	if `"`sqvar'"'=="" {
		noi di as error "sequence variable not set, use -sqset sqvar panelvar timevar"
		exit 111
	}
	
	if `"`ivar'"'=="" {
		noi di as error "panel variable not set, use -sqset sqvar panelvar timevar-"
		exit 111
	}
	
	if `"`tvar'"'=="" {
		noi di as error "time variable not set, use -sqset sqvar panelvar timevar-"
		exit 111
	}
end
	
program define Check, rclass
	local sqvar `_dta[SQis]'
	local tvar `_dta[SQtis]'
	local ivar `_dta[SQiis]'

	quietly {

		// Check Time Variable
		capture assert `tvar' < . 
		if _rc {
			noi di as error "time variable has missings"
			exit 9
		}

		tempvar control
		by `ivar' (`tvar'), sort: gen `control' = `timevar' - `timevar'[_n-1]
		sum `control', meanonly
		capture assert r(min) != r(max)
		if _rc {
			noi di as error "time variable has gaps"
			exit 9
		}

		capture assert `tvar'==int(`tvar')
		if _rc { 
			noi di as error "time variable has non integer values"
			exit 9
		}


		// Check Panel Variable
		capture assert `ivar' < . 
		if _rc {
			noi di as error "panel variable has missings"
			exit 9
		}
		capture by `ivar' `tvar': assert _n==1
		if _rc {
			noi di as error "panel variable not unique within time var"
			exit 9
		}

		// Sequence range
		sum `sqvar', meanonly
		ret scalar sqmin = r(min)
		ret scalar sqmax = r(max)
		capture assert `sqvar' <  .
		if _rc local missflag ", and missings"
		noi di as text _col(8)  "sequence variable:  " ///
		  as result "`sqvar', " r(min) " to " r(max) "`missflag'"
		
		// Panel range
		sum `ivar', meanonly
		ret scalar imin = r(min)
		ret scalar imax = r(max)
		noi di as text _col(8)  "panel variable:  " ///
		  as result "`ivar', " r(min) " to " r(max)
		
		// time range
		sum `tvar' , meanonly 
		ret scalar tmin = r(min)
		ret scalar tmax = r(max)
		noi di as text _col(8)  "time variable:  " ///
		  as result "`tvar', " r(min) " to " r(max)

	}		
		
end

