*! version 1.0.0 March 28, 2006 @ 13:42:36
*! Sumarize information on (person specific) sequences
program define sqtab
version 9
	syntax [varname(default=none)] [if] [in] ///
	  [, I(varname) T(varname) sq(varname) nosort  ranks(numlist) so ss * ]

	// Sq-Data
	capture _sqset `sq' `i' `t'
	if _rc {
		di as error "data not declared as SQ-data; use -sqset-"
		exit 9
	}

	// if/in
	if "`if'" != "" {
		tokenize "`if'", parse(" =+-*/^~!|&<>(),.")
		while "`1'" != "" {
			capture confirm variable `1'
			if !_rc {
				local iflist  "`iflist' `1'"
			}
			macro shift
		}
	}
	if "`iflist'" != "" CheckConstant `iflist', stop
	marksample touse 

	// Options
	local sort = cond("`sort'"=="nosort","","sort")

	quietly {
		
		preserve
		keep if `touse'

		if "`so'" == "so" {
			by `_dta[SQiis]' (`_dta[SQtis]'), sort: keep if `_dta[SQis]' ~= `_dta[SQis]'[_n-1]
			by `_dta[SQiis]' (`_dta[SQtis]'): replace `_dta[SQtis]' = _n
		}
		
		if "`ss'" == "ss" {
			by `_dta[SQiis]' `_dta[SQis]', sort: keep if _n == 1
			sum `_dta[SQis]', meanonly
			replace `_dta[SQtis]' = cond(r(min)<0,`_dta[SQis]'+abs(r(min)),`_dta[SQis]')
		}


		// Option ranks
	if "`ranks'" != "" {
		quietly { 
			tempfile original frequency
			save `original'
			tempvar n
			keep `_dta[SQiis]' `_dta[SQtis]' `_dta[SQis]' `touse'
			keep if `touse'
			reshape wide `_dta[SQis]', i(`_dta[SQiis]') j(`_dta[SQtis]')
			bysort `_dta[SQis]'*: gen `n' = _N
			bysort `_dta[SQis]'*: keep if _n==1
			keep `_dta[SQiis]' `n'
			KeepRanks `n', ranks(`ranks') 
			sort `_dta[SQiis]'
			save `frequency'
			use `original', clear
			sort `_dta[SQiis]'
			merge `_dta[SQiis]' using `frequency'
			keep if _merge==3
			drop _merge
		}
	}


		// Reshape to Wide
		// ---------------
	
		keep `varlist' `iflist' `_dta[SQiis]' `_dta[SQtis]' `_dta[SQis]' `n' `touse'
		reshape wide `_dta[SQis]', i(`_dta[SQiis]') j(`_dta[SQtis]')
		sort `_dta[SQis]'*

		// Generate Display-Variable
		// -------------------------
		
		tempvar pattern

		// Essential code from egen-concat
		gen str1 `pattern' = "" 
		foreach var of varlist `_dta[SQis]'* {
			capture confirm string variable `var'
			if _rc {
				replace `pattern' = `pattern' + string(`var') 
			}
			else {
				replace `pattern' = `pattern' + `var' 
			}
		}
		replace `pattern' = trim(substr(`pattern',1,length(`pattern')))

		// I want labels
		local label = ////
		  cond("`ss'"=="" & "`so'"=="","Sequence-Pattern",  ///
		  cond("`ss'"=="" & "`so'"!="","Sequence-Order", ///
		  cond("`ss'"!="" & "`so'"=="","Sequence-Stages","")))
		label variable `pattern' "`label'"
		  
	}

	// Output
	// ------

	local fweight = cond("`ranks'"=="","","[fweight=`n']")
	tab `pattern' `varlist' `fweight', `sort' `options'
		
	end
		

program CheckConstant, rclass
	syntax varlist(default=none) [, stop]
	sort `_dta[SQiis]'
	foreach var of local varlist {
		capture by `_dta[SQiis]': assert `var' == `var'[_n-1] if _n != 1
		if _rc & "`stop'" == "" {
			di as res "`var'" as text " is not constant over time; not used"
			local varlist: subinstr local varlist "`var'" "", word
		}
		if _rc & "`stop'" != "" {
			di as error "`var' is not constant over time"
			exit 9
		}
		if "`stop'" == "" {
			return local checked "`varlist'"
		}
	}
end


// Selects Ranks according to rank-Options
program KeepRanks
	syntax varname, ranks(string) 
	tempvar rank select
	sort `1'
	gen `rank' = (_N+1) - _n
	gen `select' = 0
	foreach r of local ranks {
		replace `select' = 1 if `rank'  == `r'
	}
	keep if `select'
end	
